//
//  ViewController.m
//  QuickPayExample
//

#import "ViewController.h"
#import <QuickPayLibraryF/QuickPayLibraryF.h>
#import <QuickPayLibraryF/QuickPayLibrary.h>
#import <QuickPayLibraryF/QPLCreatePaymentParameters.h>
#import <QuickPayLibraryF/QPLDefaultHeader.h>
#import <QuickPayLibraryF/QPLGetPaymentRequest.h>

@interface ViewController ()

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
}

- (IBAction)onButtonTap:(id)sender {
    // These header values are required.
    QPLDefaultHeader * headers = [[QPLDefaultHeader alloc] init];
    [headers setAuthorization: @"99357254255b6cc17e9474e8d24c9e550a8504760ab3df537a5a09192e97b63e"];
    [headers setAcceptVersion: @"v10"];
    
    // The orderId and currency parameters are required.
    QPLCreatePaymentParameters * params = [[QPLCreatePaymentParameters alloc] init];
    
    // Order Id's can only be used once. You need to change it to run the example again.
    [params setOrderId:  @"143512"];
    [params setCurrency: @"DKK"];
    
    // We can configure optional parameters
    [params setTextOnStatement:@"QuickPay Api on iOS"];
    
    // Some types are represented by NSObjects instead of NSDictionaries.
    QPLAddress * invoiceAddress = [[QPLAddress alloc]init]; // Create an address and configure it.
    [invoiceAddress setName:@"CV"];
    [invoiceAddress setCity:@"Aarhus"];
    [invoiceAddress setCountryCode:@"DNK"];
    
    [params setInvoiceAddress:invoiceAddress]; // add the address to the parameters.
    
    // Configure the request before you send it.
    QPLCreatePaymentRequest * createPaymentRequest = [[QPLCreatePaymentRequest alloc] init];
    [createPaymentRequest setParameters:params];
    [createPaymentRequest setHeader:headers];
    
    /*
     * Send the request that creates the payment on QuickPays server.
     */
    [createPaymentRequest sendRequestWithCompletionHandler:^void(NSData * data, NSURLResponse * response, NSError * error){
        
        // NSLog(@"Got response from POST /payments/ \n %@ with error %@.\n", response, error);
        // NSLog(@"DATA:\n%@\nEND DATA\n", [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
        
        if(error){
            NSLog(@"%@", error.localizedDescription);
        }
        
        NSDictionary * root   = [NSJSONSerialization JSONObjectWithData:data options:0 error:nil];
        NSNumber *  paymentId = [root objectForKey:@"id"];
        
        // These two parameters are required.
        QPLGeneratePaymentLinkParameters * linkParams = [[QPLGeneratePaymentLinkParameters alloc] init];
        [linkParams setId:      paymentId];
        [linkParams setAmount:  @123];
        
        // Configure the request before you send it.
        QPLGeneratePaymentLinkRequest * linkRequest = [[QPLGeneratePaymentLinkRequest alloc] init];
        [linkRequest setParameters: linkParams];
        [linkRequest setHeader:     headers];
        
        // Send the request.
        [linkRequest sendRequestWithCompletionHandler:^void(NSData * data, NSURLResponse * response, NSError * error){
            NSDictionary *  linkRoot = [NSJSONSerialization JSONObjectWithData:data options:0 error:nil];
            NSString *      finalUrl = [linkRoot objectForKey:@"url"];
            
            if(error){
                NSLog(@"%@", error.localizedDescription);
            }
            
            NSLog(@"Opening web view with: %@", finalUrl);
            
            /*
             * Opens a webview that shows QuickPay's page, which lets the user enter his credit card credential.
             * The view can be cancelled, or the payment can succeed or fail.
             */
            dispatch_async(dispatch_get_main_queue(), ^{
                [QuickPayLibrary openLinkInWebView:finalUrl cancelHandler:^void(){
                    /* The user pressed the "cancel" button */
                    NSLog(@"User cancelled.");
                } responseHandler:^void(NSNumber * response){ // The value of 'response' is 1 for success, and 0 for failure.
                    
                    /*
                     * Request the payment data, for example to check if everything is as you expect.
                     */
                    
                    QPLGetPaymentRequest * getRequest = [[QPLGetPaymentRequest alloc] initWithPaymentId:paymentId];
                    [getRequest setHeaders:headers];
                    [getRequest sendRequestWithCompletionHandler:^void(NSData * data, NSURLResponse * response, NSError * error){
                        
                        // NSLog(@"Got response from GET /payments/ \n %@ with error %@.\n", response, error);
                        // NSLog(@"DATA:\n%@\nEND DATA\n", [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
                        
                        if(error){
                            NSLog(@"%@", error.localizedDescription);
                        } else {
                            /*
                             * If we successfully retrieved a Payment, we can turn it into an QPLPayment
                             */
                            NSDictionary * paymentRoot = [NSJSONSerialization JSONObjectWithData:data options:0 error:nil];
                            QPLPayment * paymentFromRequest = [QPLPayment paymentFromDictionary:paymentRoot];
                            NSLog(@"Payment Acquirer: %@", [paymentFromRequest acquirer]);
                        }
                    }];
                    
                    /* The response can be failure (0) or success (1) */
                    NSLog(@"Response: %d", response.intValue);
                    
                    NSString * responseString = @"";
                    if(response.intValue == 0){
                        responseString = @"Failure";
                    } else {
                        responseString = @"Success";
                    }
                    
                    UIAlertController* alert = [UIAlertController alertControllerWithTitle:@"Response"
                                                                                   message:responseString
                                                                            preferredStyle:UIAlertControllerStyleAlert];
                    
                    UIAlertAction* defaultAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault
                                                                          handler:^(UIAlertAction * action) {}];
                    
                    [alert addAction:defaultAction];
                    [self presentViewController:alert animated:YES completion:nil];
                }];
            });
            
        }];
    }];
}

- (void)viewDidAppear:(BOOL)animated {
    
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


@end
